#include "Riostream.h"
#include "TDatime.h"
#include "TPolyMarker.h"
#include "TPolyLine.h"
#include "TLine.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TAxis.h"
#include "TH1.h"
#include "TH2.h"
#include "TLegend.h"  
#include "TFile.h"
#include "TTree.h"
#include "TStyle.h"
#include "TChain.h"
#include "TMarker.h"
#include "TROOT.h"
#include "TF1.h"
#include "TProfile.h"
#include "TStyle.h"

void HallA_style() {
  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(505);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.1);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.3);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.045,"X");
  gStyle->SetTitleOffset(1.3,"X");
  gStyle->SetLabelSize(0.045,"Y");
  gStyle->SetTitleOffset(1.3,"Y");
  gStyle->SetNdivisions(505,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);
  gStyle->SetEndErrorSize(7);
}



double CalculateFactor(double MulFactor, int MAXnum, int PaddleNumber )
{
  Double_t BB_tp_de_LE[24];
  Double_t BB_tp_de_RE[24];  
  Int_t Ndata_BB_tp_de_LE;
  Int_t Ndata_BB_tp_de_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];
  Double_t BB_tp_de_hit_bar[100];
  Double_t BB_tp_trHitIndex[100];


  TChain *t1 =new TChain("tout");
  
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3488.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3265.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3608.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3270.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3271.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3272.root");
  assert(t1);  

  t1->SetBranchAddress( "BB.tp.de.LE", &BB_tp_de_LE);
  t1->SetBranchAddress( "BB.tp.de.RE", &BB_tp_de_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.LE", &Ndata_BB_tp_de_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.RE", &Ndata_BB_tp_de_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try); 
  t1->SetBranchAddress( "BB.tp.de.hit_bar", &BB_tp_de_hit_bar);  
  t1->SetBranchAddress( "BB.tp.trHitIndex", &BB_tp_trHitIndex);




  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;
  double number = MAXnum;
  if (MAXnum<0) number = num;

  TH1F *hLE = new TH1F("hLE", "hLE", 100,0,4000);
  TH1F *hRE = new TH1F("hRE", "hRE", 100,0,4000);
  TH1F *hREminusLE = new TH1F("hREminusLE", "hREminusLE", 100,-300,300);

  TH2F *hLE_RE = new TH2F("hLE_RE", "hLE_RE", 200,0,4000,200,0,4000);


  for (int i = 0; i<number; i++)
  {
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int dEplaneHit = (int)BB_tp_de_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];
   
    if (dEplaneHit != PaddleNumber) continue;

    if (BB_tp_de_LE[PaddleNumber] < 150 || BB_tp_de_RE[PaddleNumber] < 150 ) continue;
    if ( fabs(FpY) >0.01 ) continue;

    hLE->Fill(BB_tp_de_LE[PaddleNumber]);
    hRE->Fill(BB_tp_de_RE[PaddleNumber]);
    hREminusLE->Fill(MulFactor*BB_tp_de_RE[PaddleNumber] - BB_tp_de_LE[PaddleNumber]);
    hLE_RE->Fill(BB_tp_de_LE[PaddleNumber], BB_tp_de_RE[PaddleNumber]);
 
  }

  TCanvas* c1 = new TCanvas("c1","c1",1170,660);
  c1->Divide(3,1,0.001,0.001);
  c1->cd(1);
  gPad->SetGrid();
  gPad->SetLogy();
  hRE->Draw();
  hRE->SetLineColor(kGreen);
  hLE->Draw("same");
  hLE->SetLineColor(kRed);

  c1->cd(2);
  gPad->SetGrid();
  hREminusLE->Draw();
  hREminusLE->SetLineColor(kBlue);
  TF1 *f1 = new TF1("f1", "gaus", -200, 200);
  hREminusLE->Fit("f1", "RWV");

  cout<<"Parameter 1: "<<f1->GetParameter(1)<<endl;

  c1->cd(3);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hLE_RE->Draw("colz");

 
  double result =  f1->GetParameter(1);


  delete hLE;
  delete hRE;
  delete hREminusLE;
  delete hLE_RE;
  delete f1;
  delete c1;
  delete t1;
  
  

  return result;
}



double DoMinimization(int MAXnum, int PaddleNumber)
{
  double minFac = 0.5;
  double maxFac = 1.6;
  const int MaxIter = 40;


  double tockeX[MaxIter];
  double tockeY[MaxIter];
  double tockeZ[MaxIter];
   
  double val = 1.0;
  int num = 0;
  double fac;
  
  while ( fabs(val)>0.01 && num < MaxIter )
  {
    
    fac = (minFac + maxFac)/2.0;
    val = CalculateFactor(fac, MAXnum, PaddleNumber);
    if (val < 0.0){
      minFac = fac;
    }
    else{ 
       maxFac = fac;
    }

    tockeX[num] = num;
    tockeY[num] = fac;
    tockeZ[num] = val+1.0;
    num++;

  }


  TCanvas* c2 = new TCanvas("c2","c2",1170,660);
  c2->Divide(2,1,0.001,0.001);
  c2->cd(1);
  gPad->SetGrid();
  TGraph *gr = new TGraph(num, tockeX, tockeY);
  gr->Draw("APL");
  gr->SetLineColor(kRed);

  TString figurename;
  figurename.Form("Mult. factor for dE-paddle #%d",PaddleNumber);
  gr->SetTitle(figurename.Data());
  gr->GetXaxis()->SetTitle("Iteration [1]");
  gr->GetYaxis()->SetTitle("R-Mult. factor [1]");
 
  c2->cd(2);
  gPad->SetGrid();
  gPad->SetLogy();
  TGraph *gr2 = new TGraph(num, tockeX, tockeZ);
  gr2->Draw("APL");
  gr2->SetLineColor(kBlue);

  figurename.Form("Offset from center in dE-paddle #%d",PaddleNumber);
  gr2->SetTitle(figurename.Data());
  gr2->GetXaxis()->SetTitle("Iteration [1]");
  gr2->GetYaxis()->SetTitle("1+Offset [ADC channel]");


  figurename.Form("figure_dEPaddle_%d_OffsetMinimization.png",PaddleNumber);
  c2->Print(figurename.Data()); 


  cout<<"Final correction Factor: "<<fac<<endl;
/*
  delete gr;
  delete gr2;
  delete c2;
*/
  return fac;
  

}

Double_t fitdiff(Double_t *x, Double_t *par)
{
  Double_t G = par[0];
  Double_t L1 = par[1];
  Double_t L2 = par[2];

  return -1.0*G*(exp(L1*x[0]) - exp(-1.0*L2*x[0]));

}

double DoResults(double MulFactor, int MAXnum, int PaddleNumber, double *FitParameters)
{
  Double_t BB_tp_de_LE[24];
  Double_t BB_tp_de_RE[24];  
  Int_t Ndata_BB_tp_de_LE;
  Int_t Ndata_BB_tp_de_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];
  Double_t BB_tp_de_hit_bar[100];
  Double_t BB_tp_trHitIndex[100];

  Double_t BB_tp_e_LE[24];
  Double_t BB_tp_e_RE[24];  
  Double_t BB_tp_e_hit_bar[100];



  TChain *t1 =new TChain("tout");
  
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3270.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3271.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3272.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3273.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3274.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3275.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3276.root");

  assert(t1);  

  t1->SetBranchAddress( "BB.tp.de.LE", &BB_tp_de_LE);
  t1->SetBranchAddress( "BB.tp.de.RE", &BB_tp_de_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.LE", &Ndata_BB_tp_de_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.RE", &Ndata_BB_tp_de_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try); 
  t1->SetBranchAddress( "BB.tp.de.hit_bar", &BB_tp_de_hit_bar);  
  t1->SetBranchAddress( "BB.tp.trHitIndex", &BB_tp_trHitIndex);

  t1->SetBranchAddress( "BB.tp.e.LE", &BB_tp_e_LE);
  t1->SetBranchAddress( "BB.tp.e.RE", &BB_tp_e_RE);
  t1->SetBranchAddress( "BB.tp.e.hit_bar", &BB_tp_e_hit_bar);  


  const int NUM = 23;
  Double_t yCuts[NUM] = {-0.22, -0.2, -0.18, -0.16, -0.14, -0.12, -0.10, -0.08, -0.06, -0.04, -0.02, 
                          0.0, 0.02, 0.04, 0.06, 0.08,0.1,0.12, 0.14, 0.16, 0.18, 0.2, 0.22};

  TH1F *hLE[NUM];
  TH1F *hRE[NUM];
  TH1F *hREminusLE[NUM];
  TH2F *hLE_RE[NUM];
  TCanvas* cY[NUM];
  TF1 *f1Y[NUM] ;

  Double_t resY[NUM];
  Double_t resOffset[NUM];
  Double_t resSigmaY[NUM];
  Double_t resSigmaOffset[NUM];


  for (int M = 0; M<NUM; M++){

    TString name;
    name.Form("hLE_YCut_%d",M);
    hLE[M] = new TH1F(name.Data(), name.Data(), 100,0,4000);
    
    name.Form("hRE_YCut_%d",M);
    hRE[M] = new TH1F(name.Data(), name.Data(), 100,0,4000);

    name.Form("hREminusLE_YCut_%d",M);
    hREminusLE[M] = new TH1F(name.Data(), name.Data(), 100,-900,900);

    name.Form("hLE_RE_YCut_%d",M);   
    hLE_RE[M] = new TH2F(name.Data(), name.Data(), 200,0,4000,200,0,4000);
 

  }


   TH2F *hLEREdiff  = new TH2F("hLEREdiff","hLEREdiff",200,-0.25,0.25, 300,-2000.0, 2000.0);


  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;
  double number = MAXnum;
  if (MAXnum<0) number = num;

  for (int i = 0; i<number; i++){
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int dEplaneHit = (int)BB_tp_de_hit_bar[trHitIndex];
    int EplaneHit = (int)BB_tp_e_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];
    
    

    if (dEplaneHit!=PaddleNumber) continue;

    for (int M = 0; M<NUM; M++){
      if (BB_tp_de_LE[PaddleNumber] < 150 || BB_tp_de_RE[PaddleNumber] < 150 ) continue;
      if ( fabs(FpY - yCuts[M]) >0.01 ) continue;

      hLE[M]->Fill(BB_tp_de_LE[PaddleNumber]);
      hRE[M]->Fill(BB_tp_de_RE[PaddleNumber]);
      hREminusLE[M]->Fill(MulFactor*BB_tp_de_RE[PaddleNumber] - BB_tp_de_LE[PaddleNumber]);
      hLE_RE[M]->Fill(BB_tp_de_LE[PaddleNumber], BB_tp_de_RE[PaddleNumber]);
   
      hLEREdiff->Fill(FpY,(MulFactor*BB_tp_de_RE[PaddleNumber] - BB_tp_de_LE[PaddleNumber]));
     
    }
 
  }

  for (int M = 0; M<NUM; M++){
    TString name;
    name.Form("CY_%d",M);
    cY[M] = new TCanvas(name.Data(),name.Data(),1170,660);
    cY[M]->Divide(3,1,0.001,0.001);
    cY[M]->cd(1);
    gPad->SetGrid();
    gPad->SetLogy();
    hRE[M]->Draw();
    hRE[M]->SetLineColor(kGreen);
    hLE[M]->Draw("same");
    hLE[M]->SetLineColor(kRed);

    cY[M]->cd(2);
    gPad->SetGrid();
    hREminusLE[M]->Draw();
    hREminusLE[M]->SetLineColor(kBlue);
    name.Form("f1Y_%d",M);
    f1Y[M] = new TF1(name.Data(), "gaus", -900, 900);
    hREminusLE[M]->Fit(name.Data(), "RWV");

    cout<<"Parameter <mean>: "<<f1Y[M]->GetParameter(1)<<endl;
    cout<<"Parameter <sigma>: "<<f1Y[M]->GetParameter(2)<<endl;

    resY[M] = yCuts[M];
    resOffset[M] = f1Y[M]->GetParameter(1);
    resSigmaOffset[M] = f1Y[M]->GetParameter(2);
    resSigmaY[M] = 0.01;

    cY[M]->cd(3);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hLE_RE[M]->Draw("colz");

    name.Form("figure_dEPaddle_%d_YCut%d.png",PaddleNumber, M);
    cY[M]->Print(name.Data());

  }
  

  TCanvas* cRes = new TCanvas("cRes","cRes",1170,660);
  cRes->Divide(2,1,0.001,0.001);
  cRes->cd(1);
  gPad->SetGrid();
  TGraphErrors *gr = new TGraphErrors(NUM, resY, resOffset, resSigmaY, resSigmaOffset);
  gr->Draw("AP");
  gr->SetMarkerStyle(20);
  gr->SetMarkerColor(kRed);
 
  TString figurename;
  figurename.Form("Y Dependence for dE-paddle #%d",PaddleNumber);
  gr->SetTitle(figurename.Data());
  gr->GetXaxis()->SetTitle("Y-offset [m]");
  gr->GetYaxis()->SetTitle("ADC-offset [channel]");

  TF1 *f1 = new TF1("f1", fitdiff, -0.25, 0.25, 3);
  f1->SetParNames("G0","L1","L2");
  f1->SetParLimits(0, 0.0, 500.0);
  //f1->SetParLimits(1, 0.0, 7.0);
  //f1->SetParLimits(2, 0.0, 7.0);
  f1->FixParameter(1, FitParameters[1]);
  f1->FixParameter(2, FitParameters[2]);


  gr->Fit("f1","RWV");

  FitParameters[0] = f1->GetParameter(0);
  FitParameters[1] = f1->GetParameter(1);
  FitParameters[2] = f1->GetParameter(2);
    
  cRes->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hLEREdiff->Draw("colz");
  f1->Draw("same");

  figurename.Form("figure_dEPaddle_%d_YDep.png",PaddleNumber);
  cRes->Print(figurename.Data());



  for (int M = 0; M<NUM; M++){
    delete hLE[M];
    delete hRE[M];
    delete hREminusLE[M];
    delete hLE_RE[M];
    delete f1Y[M];
    delete cY[M];
  }
   
  delete hLEREdiff;
  delete f1;
  delete gr;
  delete cRes;
  delete t1;


}

Double_t fitf(Double_t *x, Double_t *par)
{
  Double_t G = par[0];
  Double_t L1 = par[1];
  Double_t L2 = par[2];

  return G*(exp(L1*x[0]) + exp(-1.0*L2*x[0]))/2.0;

}

double MeanYDependence(double MulFactor, int MAXnum, int PaddleNumber, double *ResParams)
{
  Double_t BB_tp_e_LE[24];
  Double_t BB_tp_e_RE[24];  
  Int_t Ndata_BB_tp_e_LE;
  Int_t Ndata_BB_tp_e_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];
  Double_t BB_tp_e_hit_bar[100];
  Double_t BB_tp_trHitIndex[100];


  TChain *t1 =new TChain("tout");

  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3270.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3271.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3272.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3273.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3274.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3275.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3276.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3277.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3278.root");
  assert(t1);  

  t1->SetBranchAddress( "BB.tp.e.LE", &BB_tp_e_LE);
  t1->SetBranchAddress( "BB.tp.e.RE", &BB_tp_e_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.e.LE", &Ndata_BB_tp_e_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.e.RE", &Ndata_BB_tp_e_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try);
  t1->SetBranchAddress( "BB.tp.e.hit_bar", &BB_tp_e_hit_bar);  
  t1->SetBranchAddress( "BB.tp.trHitIndex", &BB_tp_trHitIndex);
  
  const int NUM = 23;
  Double_t yCuts[NUM] = {-0.22, -0.2, -0.18, -0.16, -0.14, -0.12, -0.10, -0.08, -0.06, -0.04, -0.02, 
                          0.0, 0.02, 0.04, 0.06, 0.08,0.1,0.12, 0.14, 0.16, 0.18, 0.2, 0.22};

  Double_t CutSize = 0.01;

  Double_t resY[NUM];
  Double_t resADCMean[NUM];
  Double_t resSigmaY[NUM];
  Double_t resSigmaADCMean[NUM];
  TH1F *hADCE_FpYDep[NUM];
  TCanvas *cY[NUM] ;
  TF1 *fitfun[NUM]; 


  for(int i = 0; i<NUM; i++){

    TString name;
    name.Form("hADCE_FpYDep_%d",i);
    hADCE_FpYDep[i] = new TH1F(name.Data(), name.Data(), 100,0,2500);
    
  }


  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;
  double number = MAXnum;
  if (MAXnum<0) number = num;

  for (int i = 0; i<number; i++){
  //for (int i = 0; i<100; i++){
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int EplaneHit = (int)BB_tp_e_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];

    if (EplaneHit!=PaddleNumber) continue;
    
    for(int j = 0; j<NUM; j++){ 
      if ( fabs(FpY-yCuts[j]) > CutSize ) continue;
      hADCE_FpYDep[j]->Fill(sqrt(MulFactor*BB_tp_e_RE[PaddleNumber]  *  BB_tp_e_LE[PaddleNumber]));

    }


  }

  for (int i = 0; i<NUM; i++){
    TString name;
    name.Form("CY_%d",i);  
    cY[i] = new TCanvas(name.Data(),name.Data(),1170,660);
    cY[i]->Divide(1,1,0.001,0.001);
    cY[i]->cd(1);
    gPad->SetGrid(); 
    hADCE_FpYDep[i]->Draw();
    name.Form("fitfun_%d",i);  
    //fitfun[i] = new TF1(name.Data(), "gaus", 0, 2500);
    fitfun[i] = new TF1(name.Data(), "landau", 0, 2500);
    hADCE_FpYDep[i]->Fit(name.Data(), "RWV");

    resY[i] = yCuts[i];
    resADCMean[i] = fitfun[i]->GetParameter(1);
    resSigmaY[i] = CutSize;
    resSigmaADCMean[i]= fitfun[i]->GetParError(1);
    //resSigmaADCMean[i]= fitfun[i]->GetParError(1);

    name.Form("figure_EPaddle_%d_YCorrectionFactor_YCut_%d.png",PaddleNumber, i);
    cY[i]->Print(name.Data());
    
    delete fitfun[i];
    delete hADCE_FpYDep[i];
    delete cY[i];
  }
  

  TCanvas* cRes = new TCanvas("cRes","cRes",1170,660);
  cRes->Divide(1,1,0.001,0.001);
  cRes->cd(1);
  gPad->SetGrid();
  TGraphErrors *gr = new TGraphErrors(NUM, resY, resADCMean, resSigmaY, resSigmaADCMean);
  gr->Draw("AP");
  gr->SetMarkerStyle(20);
  gr->SetMarkerColor(kRed);
  TF1 *f1 = new TF1("f1", fitf, -0.25, 0.25, 3);
  f1->SetParNames("G0","L1","L2");
  f1->SetParLimits(0, 600.0, 1050.0);
  f1->SetParLimits(1, 3.0, 5.0);
  f1->SetParLimits(2, 3.0, 5.0);
  //f1->FixParameter(0, 176.95);
  //f1->FixParameter(1, 1.71273);
  //f1->FixParameter(2, 1.82481);
  gr->Fit("f1","RWWV");

  ResParams[0] = f1->GetParameter(0);
  ResParams[1] = f1->GetParameter(1);
  ResParams[2] = f1->GetParameter(2);


  TString figurename;
  figurename.Form("Y Dependence of <ADC> for E-paddle #%d",PaddleNumber);
  gr->SetTitle(figurename.Data());
  gr->GetXaxis()->SetTitle("Y-offset [m]");
  gr->GetYaxis()->SetTitle("<ADC> [channel]");
 
  figurename.Form("figure_EPaddle_%d_YCorrectionFactorsFit.png",PaddleNumber);
  cRes->Print(figurename.Data());




    
  TH2F *hADCEvsFpYPaddle = new TH2F("hADCEvsFpYPaddle", "hADCEvsFpYPaddle", 500,-0.3,0.3,500,0,4500);
  TH2F *hADCEvsFpYPaddleCorrected = new TH2F("hADCEvsFpYPaddleCorrected", "hADCEvsFpYPaddleCorrected", 500,-0.3,0.3,500,0,4500);

  for (int i = 0; i<number; i++){
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  Equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int EplaneHit = (int)BB_tp_e_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];

    if (EplaneHit!=PaddleNumber) continue;

    
    //double EmeanADCValue = sqrt(MulFactor*BB_tp_e_RE[EplaneHit]*BB_tp_e_LE[EplaneHit]);
    double EmeanADCValue = 0.5*(MulFactor*BB_tp_e_RE[EplaneHit]+BB_tp_e_LE[EplaneHit]);
    double EmeanADCValueCorrected = EmeanADCValue - (fitf(&FpY, ResParams) - ResParams[0]);


    hADCEvsFpYPaddle->Fill(FpY,EmeanADCValue);
    hADCEvsFpYPaddleCorrected->Fill(FpY,EmeanADCValueCorrected);
   
  }

  TLine *LimitLine = new TLine(-0.3,  ResParams[0], 0.3,  ResParams[0]);
  LimitLine->SetLineColor(kRed);
  LimitLine->SetLineWidth(1.5);
  

  TCanvas *cCorr = new TCanvas("cCorr","cCorr",660,660);
  cCorr->Divide(1,2,0.001,0.001);
  cCorr->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hADCEvsFpYPaddle->Draw("colz");
  LimitLine->Draw("same");
  f1->Draw("same");


  cCorr->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hADCEvsFpYPaddleCorrected->Draw("colz");
  LimitLine->Draw("same");
  

  figurename.Form("figure_EPaddle_%d_YCorrected.png",PaddleNumber);
  cCorr->Print(figurename.Data());


/*
  delete hADCdEvsFpYPaddle;
  delete hADCdEvsFpYPaddleCorrected;
  delete LimitLine;
  delete cCorr;
  delete gr;
  delete f1;
  delete cRes;
  delete t1;
*/

    
}






void  EplaneCalibrationExp()
{
  HallA_style();



  //DoMinimization(-1, 1);

  double fac = 0.980072;
  double CorrectionParameters[3];
 //MeanYDependence(0.98266, -1, 7, CorrectionParameters);
  MeanYDependence(fac, -1, 6, CorrectionParameters);

/*

 //MeanYDependenceLowLimit(1.01529, -1, 1, CorrectionParameters);
 cout<<"Correction parameters: [G]: "<<CorrectionParameters[0]<<", [L1]: "<<CorrectionParameters[1]<<", [L2]: "<<CorrectionParameters[2]<<endl;
 DoResults(fac, -1, 1,CorrectionParameters);
 cout<<"Correction parameters: [G2]: "<<CorrectionParameters[0]<<endl;
*/


 
/*
  for (int i = 20; i<20+1; i++){

    ofstream outFile;
    TString filename;
    filename.Form("dEplaneCorrectionFactors_PaddleNo%d.dat", i);
    outFile.open(filename.Data());

    //double fac = DoMinimization(-1, i);
    double fac = 0.923846;
    cout<<"Factor for paddle: "<<i<<" is: "<<fac<<endl;

  
    double CorrectionParameters[3] = {-1,-1, -1};
 
    MeanYDependence(fac, -1, i, CorrectionParameters);
    cout<<"Correction parameters: [G]: "<<CorrectionParameters[0]<<", [L1]: "<<CorrectionParameters[1]<<", [L2]: "<<CorrectionParameters[2]<<endl;
    double FitParameters[3] = {CorrectionParameters[0], CorrectionParameters[1], CorrectionParameters[2]};
    DoResults(fac, -1, i,FitParameters);
     cout<<"Fit parameters: [fG]: "<<FitParameters[0]<<", [fL1]: "<<FitParameters[1]<<", [fL2]: "<<FitParameters[2]<<endl;

    outFile<<i<<"  "<<fac<<"  "<<FitParameters[0]<<"  "<<FitParameters[1]<<"  "<<FitParameters[2]<<"  "<<CorrectionParameters[0]<<"  "<<CorrectionParameters[1]<<"  "<<CorrectionParameters[2]<<endl;



    outFile.close();

  }

*/

}

