#include "Riostream.h"
#include "TDatime.h"
#include "TPolyMarker.h"
#include "TPolyLine.h"
#include "TLine.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TAxis.h"
#include "TH1.h"
#include "TH2.h"
#include "TLegend.h"  
#include "TFile.h"
#include "TTree.h"
#include "TStyle.h"
#include "TChain.h"
#include "TMarker.h"
#include "TROOT.h"
#include "TF1.h"
#include "TProfile.h"
#include "TStyle.h"

void HallA_style() {
  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(505);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.1);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.3);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.045,"X");
  gStyle->SetTitleOffset(1.3,"X");
  gStyle->SetLabelSize(0.045,"Y");
  gStyle->SetTitleOffset(1.3,"Y");
  gStyle->SetNdivisions(505,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);
  gStyle->SetEndErrorSize(7);
}


double DoResults(int MAXnum)
{
  double ERightGainCorrections[24] = {	1.01282,
					0.998535,
					1.00527,
					0.987726,
					1.00142,
					0.985367,
					0.980072,
					0.993866,
					1.0119,
					0.979065,
					0.987534,
					1.01997,
					0.99292,
					0.983154,
					0.998004,
					0.996338,
					0.998474,
					1.0009,
					0.996709,
					0.97673,
					1.01855,
					0.996763,
					1.00377, 
					0.842193};

double dERightGainCorrections[24] = {	
0.953796,
1.01506,
0.958991,
0.986622,
0.999998,
0.946187,
0.949359,
0.979269,
0.978967,
0.974603,
0.970407,
1.00247,
0.992328,
1.16679,
0.830457,
1.04745,
1.06705,
1.0365,
0.908523,
1.07288,
0.998489,
0.851028,
0.882453,
1.00041
};


double dEFinalCorrections[24] = {	
1.0,
1.8,
1.84,
1.7,
1.648342, //4
1.838235, //5
1.522997, //6
1.452133, //7
1.67672702,
1.8341892,
1.67841558,
1.6149871,
1.6350556,
1.388638,
1.9833399,
1.58579131,
1.47,
1.90223,
1.0,
1.0,
1.0,
1.0,
1.0,
1.0
};

double EFinalCorrections[24] = {	
1.0, 
0.98,
1.0,
1.16034,
1.117886,
1.15972,
1.124744,
1.0597301,
1.107754,
1.190476,
1.1178861,
1.12474,
1.160338,
1.1332325,//1.0848126,
1.101101,
1.10775,
1.085385, //1.05973,
1.15304,
1.0,
1.0,
1.0,
1.0,
1.0,
1.0
};

  Double_t xC[9] = {500.0, 852.0, 1200.0, 1500.0, 2100.0, 1356.0, 1000.0, 762.0, 0.0};
  Double_t yC[9] = {1750.0, 1350.0, 1130.0, 970.0, 770.0, 625.0, 500.0, 400.0, 0.0};
  TPolyLine *pline1 = new TPolyLine(9,xC,yC);
  pline1->SetFillColor(38);
  pline1->SetLineColor(2);
  pline1->SetLineWidth(2);

  TF1 *funPoly4 = new TF1("funPoly4","350.0*(1.0 + 150.0*pow(x,4))",-0.25,0.24);
  funPoly4->SetLineColor(2);
  funPoly4->SetLineWidth(2);


  
  Double_t BB_tp_e_LE[24];
  Double_t BB_tp_e_RE[24];  
  Int_t Ndata_BB_tp_e_LE;
  Int_t Ndata_BB_tp_e_RE;  
  Double_t BB_tp_de_LE[24];
  Double_t BB_tp_de_RE[24];  
  Int_t Ndata_BB_tp_de_LE;
  Int_t Ndata_BB_tp_de_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];
  Double_t BB_tp_trHitIndex[100];
  Double_t BB_tp_e_hit_bar[100];
  Double_t BB_tp_de_hit_bar[100];
  Double_t BB_tr_ph[100];
  Double_t DL_t3[100];


  TChain *t1 =new TChain("tout");
  
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3488.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_1518.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_2164.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_2413.root");


  assert(t1);  

  t1->SetBranchAddress( "BB.tp.e.LE", &BB_tp_e_LE);
  t1->SetBranchAddress( "BB.tp.e.RE", &BB_tp_e_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.e.LE", &Ndata_BB_tp_e_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.e.RE", &Ndata_BB_tp_e_RE);
  t1->SetBranchAddress( "BB.tp.de.LE", &BB_tp_de_LE);
  t1->SetBranchAddress( "BB.tp.de.RE", &BB_tp_de_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.LE", &Ndata_BB_tp_de_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.RE", &Ndata_BB_tp_de_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try); 
  t1->SetBranchAddress( "BB.tp.e.hit_bar", &BB_tp_e_hit_bar);
  t1->SetBranchAddress( "BB.tp.de.hit_bar", &BB_tp_de_hit_bar);
  t1->SetBranchAddress( "BB.tp.trHitIndex", &BB_tp_trHitIndex);
  t1->SetBranchAddress( "BB.tr.ph", &BB_tr_ph);
  t1->SetBranchAddress( "DL.t3", &DL_t3);  


 

  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;
  double number = MAXnum;
  if (MAXnum<0) number = num;

  TString name;
  name.Form("hEplaneGain");   
  TH2F *hEplaneGain = new TH2F(name.Data(), name.Data(), 24,0,24,200,0,4000);
  name.Form("hdEplaneGain");   
  TH2F *hdEplaneGain = new TH2F(name.Data(), name.Data(), 24,0,24,200,0,4000);
  name.Form("hEdEHitPlane");   
  TH2F *hEdEHitPlane = new TH2F(name.Data(), name.Data(), 24,0,24,24,0,24);

  
  
  TH2F *hADCEdEPaddle[24];
  TH2F *hADCEdEp1Paddle[24];
  TH1F *hADCEPaddle[24];
  TH1F *hADCdEPaddle[24];
  TH2F *hADCEvsFpYPaddle[24];
  TH2F *hADCdEvsFpYPaddle[24];
 
  TH2F *hADCE_DLt3_Paddle[24];
  TH1F *hDLt3_Paddle[24];
 

  for (int i = 0; i<24; i++){
    name.Form("hADCEdEPaddle%d",i);   
    hADCEdEPaddle[i] = new TH2F(name.Data(), name.Data(), 500,0,3000,500,0,2500);
    name.Form("hADCEdEp1Paddle%d",i);   
    hADCEdEp1Paddle[i] = new TH2F(name.Data(), name.Data(), 500,0,3000,500,0,2500);
    name.Form("ADCEPaddle%d",i);   
    hADCEPaddle[i] = new TH1F(name.Data(), name.Data(), 500,0,3000);
    name.Form("hADCdEPaddle%d",i);   
    hADCdEPaddle[i] = new TH1F(name.Data(), name.Data(), 120,0,1000);

    name.Form("hADCEvsFpYPaddle%d",i);   
    hADCEvsFpYPaddle[i] = new TH2F(name.Data(), name.Data(), 500,-0.3,0.3,500,0,3000);
    name.Form("hADCdEvsFpYPaddle%d",i);   
    hADCdEvsFpYPaddle[i] = new TH2F(name.Data(), name.Data(), 500,-0.3,0.3,500,0,2500);

   
    name.Form("hADCE_DLt3_Paddle%d",i);   
    hADCE_DLt3_Paddle[i] = new TH2F(name.Data(), name.Data(), 120,700,900,500,0,3000);

    name.Form("hDLt3_Paddle%d",i);   
    hDLt3_Paddle[i] = new TH1F(name.Data(), name.Data(), 120,700,900);


  }

  for (int i = 0; i<number; i++){
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

    if (fabs(BB_tp_try[0])>0.1) continue;

    // which hit in TP is consistent with the track in MWDC? I can choose index [0], because I consider only those
    // events with one track in chambers.
    int trHitIndex = (int) BB_tp_trHitIndex[0];
    // first we check which plane was hit. 
    int EplaneHit = (int)BB_tp_e_hit_bar[trHitIndex];
    int dEplaneHit = (int)BB_tp_de_hit_bar[trHitIndex];
    // to be able to calibrate bars, I need to know which bar exactly I am dealing with.
    if ((EplaneHit!=dEplaneHit-1) && (EplaneHit!=dEplaneHit)) continue; 
    hEdEHitPlane->Fill(EplaneHit, dEplaneHit);

    double EmeanADCValue = EFinalCorrections[EplaneHit]*0.5*(ERightGainCorrections[EplaneHit]*BB_tp_e_RE[EplaneHit]+BB_tp_e_LE[EplaneHit]);
    
    double dEmeanADCValueBLA = dEFinalCorrections[dEplaneHit]*0.5*(dERightGainCorrections[dEplaneHit]*BB_tp_de_RE[dEplaneHit]+BB_tp_de_LE[dEplaneHit]);


    
    double correction  = 1.0 ;//+ pow(BB_tp_try[0],4)*100.0;
 
    double dEmeanADCValue = dEmeanADCValueBLA/correction;

    hADCE_DLt3_Paddle[EplaneHit]->Fill(DL_t3[0], EmeanADCValue);
    hDLt3_Paddle[EplaneHit]->Fill(DL_t3[0]);



      
    //if (EmeanADCValue < 200.0 || dEmeanADCValue <200.0) continue;
    
    hEplaneGain->Fill(1.0*EplaneHit, EmeanADCValue);
    hdEplaneGain->Fill(1.0*dEplaneHit, dEmeanADCValue);
       

    hADCEvsFpYPaddle[EplaneHit]->Fill(BB_tp_try[0], EmeanADCValue ); 
    hADCdEvsFpYPaddle[dEplaneHit]->Fill(BB_tp_try[0], dEmeanADCValue);
       
    if ((EplaneHit==dEplaneHit-1)) hADCEdEp1Paddle[EplaneHit]->Fill(EmeanADCValue,dEmeanADCValue);
    if ((EplaneHit==dEplaneHit)) hADCEdEPaddle[EplaneHit]->Fill(EmeanADCValue,dEmeanADCValue);


    hADCEPaddle[EplaneHit]->Fill(EmeanADCValue);
    if (fabs(EmeanADCValue-1000.0)<30.0) hADCdEPaddle[dEplaneHit]->Fill(dEmeanADCValue);     

  }


  name.Form("C0");
  TCanvas *c0 = new TCanvas(name.Data(),name.Data(),1170,660);
  c0->Divide(1,1,0.001,0.001);
  c0->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  //gPad->SetLogz();
  hEdEHitPlane->Draw("colz");

  name.Form("figure_EdE_hitPaddle.png");
  c0->Print(name.Data());


  name.Form("C1");
  TCanvas *c1 = new TCanvas(name.Data(),name.Data(),1170,660);
  c1->Divide(1,1,0.001,0.001);
  c1->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  //gPad->SetLogz();
  hEplaneGain->Draw("colz");

  name.Form("figure_Eplane_Gain.png");
  c1->Print(name.Data());


  name.Form("C2");
  TCanvas *c2 = new TCanvas(name.Data(),name.Data(),1170,660);
  c2->Divide(1,1,0.001,0.001);
  c2->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  //gPad->SetLogz();
  hdEplaneGain->Draw("colz");

  name.Form("figure_dEplane_Gain.png");
  c2->Print(name.Data());


  int PaddleNum = 16;


  TH2F *hADCEdEPaddleSum = new TH2F("hADCEdEPaddleSum", "hADCEdEPaddleSum", 500,0,3000,500,0,2500);
  TCanvas *c3[24];
  for (int i =4; i<13+1; i++){ 
  //for (int i =PaddleNum; i<PaddleNum+1; i++){ 
    name.Form("C3_%d",i);
    c3[i] = new TCanvas(name.Data(),name.Data(),1170,660);
    c3[i]->Divide(2,1,0.001,0.001);
    c3[i]->cd(1);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hADCEdEPaddle[i]->Draw("colz");
    pline1->Draw("same");
    c3[i]->cd(2);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hADCEdEp1Paddle[i]->Draw("colz");
    pline1->Draw("same");

    name.Form("figure_EdE_paddle%d.png",i);
    c3[i]->Print(name.Data());


    hADCEdEPaddleSum->Add(hADCEdEPaddle[i],1);
    hADCEdEPaddleSum->Add(hADCEdEp1Paddle[i],1);
  }


  TCanvas *c3a = new TCanvas("c3a","c3a",660,660);
  c3a->Divide(1,1,0.001,0.001);
  c3a->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hADCEdEPaddleSum->Draw("colz");

  name.Form("figure_EdE_sum_plot.png");
  c3a->Print(name.Data());



  TCanvas *c4[24];
  TLine *UpperLimitLine[24];
  TF1 *fitfun[24];
  //for (int i = PaddleNum; i<PaddleNum+1; i++){
  for (int i = 4; i<13+1; i++){  
    name.Form("C4_%d",i);
    c4[i] = new TCanvas(name.Data(),name.Data(),1170,660);
    c4[i]->Divide(2,1,0.001,0.001);
    c4[i]->cd(1);
    gPad->SetGrid();
    gPad->SetLogy();
    hADCEPaddle[i]->Draw();
    UpperLimitLine[i] = new TLine(2200, 0.0, 2200, 10000);
    UpperLimitLine[i]->SetLineColor(kRed);
    UpperLimitLine[i]->SetLineWidth(1.5);
    UpperLimitLine[i]->Draw("same");

    c4[i]->cd(2);
    gPad->SetGrid();
    //gPad->SetLogy();
    hADCdEPaddle[i]->Draw();
    name.Form("fitfun_%d",i);  
    fitfun[i] = new TF1(name.Data(), "gaus", 0, 10000);
    hADCdEPaddle[i]->Fit(name.Data(), "RWV");
    fitfun[i]->SetLineColor(kBlue);

    name.Form("figure_EdE_calibration_paddle%d.png",i);
    c4[i]->Print(name.Data());
  }
 
  TCanvas *c5[24];

  //for (int i = PaddleNum; i<PaddleNum+1; i++){
  for (int i = 4; i<13+1; i++){  
    name.Form("C5_%d",i);
    c5[i] = new TCanvas(name.Data(),name.Data(),1170,660);
    c5[i]->Divide(1,2,0.001,0.001);
    c5[i]->cd(1);
    gPad->SetGrid();
    //gPad->SetLogy();
    gStyle->SetPalette(1);
    hADCEvsFpYPaddle[i]->Draw("colz");
    c5[i]->cd(2);
    gPad->SetGrid();
    //gPad->SetLogy();
    gStyle->SetPalette(1);
    hADCdEvsFpYPaddle[i]->Draw("colz");
 
    funPoly4->Draw("same");  

    name.Form("figure_EdE_vs_FpY_paddle%d.png",i);
    c5[i]->Print(name.Data());

  }


  TCanvas *c6[24];
  for (int i = PaddleNum; i<PaddleNum+1; i++){
  //for (int i = 4; i<15+1; i++){  
    name.Form("C6_%d",i);
    c6[i] = new TCanvas(name.Data(),name.Data(),1170,660);
    c6[i]->Divide(1,2,0.001,0.001);
    c6[i]->cd(1);
    gPad->SetGrid();
    //gPad->SetLogy();
    gStyle->SetPalette(1);
    hADCE_DLt3_Paddle[i]->Draw("colz");

    c6[i]->cd(2);
    gPad->SetGrid();
    gPad->SetLogy();
    hDLt3_Paddle[i]->Draw("");

    name.Form("figure_E_vs_DLt3_paddle%d.png",i);
    c6[i]->Print(name.Data());

  }

 

 

}



void EdECalibration()
{
 HallA_style();
 DoResults(-1);

}

