#include "Riostream.h"
#include "TDatime.h"
#include "TPolyMarker.h"
#include "TPolyLine.h"
#include "TLine.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TAxis.h"
#include "TH1.h"
#include "TH2.h"
#include "TLegend.h"  
#include "TFile.h"
#include "TTree.h"
#include "TStyle.h"
#include "TChain.h"
#include "TMarker.h"
#include "TROOT.h"
#include "TF1.h"
#include "TProfile.h"
#include "TStyle.h"

void HallA_style() {
  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(505);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.1);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.3);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.045,"X");
  gStyle->SetTitleOffset(1.3,"X");
  gStyle->SetLabelSize(0.045,"Y");
  gStyle->SetTitleOffset(1.3,"Y");
  gStyle->SetNdivisions(505,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);
  gStyle->SetEndErrorSize(7);
}



double CalculateFactor(double MulFactor, int MAXnum, int PaddleNumber )
{
  Double_t BB_tp_e_LE[24];
  Double_t BB_tp_e_RE[24];  
  Int_t Ndata_BB_tp_e_LE;
  Int_t Ndata_BB_tp_e_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];


  TChain *t1 =new TChain("tout");
  
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3488.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  assert(t1);  

  t1->SetBranchAddress( "BB.tp.e.LE", &BB_tp_e_LE);
  t1->SetBranchAddress( "BB.tp.e.RE", &BB_tp_e_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.e.LE", &Ndata_BB_tp_e_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.e.RE", &Ndata_BB_tp_e_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try); 




  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;
  double number = MAXnum;
  if (MAXnum<0) number = num;

  TH1F *hLE = new TH1F("hLE", "hLE", 100,0,4000);
  TH1F *hRE = new TH1F("hRE", "hRE", 100,0,4000);
  TH1F *hREminusLE = new TH1F("hREminusLE", "hREminusLE", 100,-300,300);

  TH2F *hLE_RE = new TH2F("hLE_RE", "hLE_RE", 200,0,4000,200,0,4000);


  for (int i = 0; i<number; i++)
  {
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

    if (BB_tp_e_LE[PaddleNumber] < 500 || BB_tp_e_RE[PaddleNumber] < 500 ) continue;
    if ( fabs(BB_tp_try[0]) >0.01 ) continue;

    hLE->Fill(BB_tp_e_LE[PaddleNumber]);
    hRE->Fill(BB_tp_e_RE[PaddleNumber]);
    hREminusLE->Fill(MulFactor*BB_tp_e_RE[PaddleNumber] - BB_tp_e_LE[PaddleNumber]);
    hLE_RE->Fill(BB_tp_e_LE[PaddleNumber], BB_tp_e_RE[PaddleNumber]);
 
  }

  TCanvas* c1 = new TCanvas("c1","c1",1170,660);
  c1->Divide(3,1,0.001,0.001);
  c1->cd(1);
  gPad->SetGrid();
  gPad->SetLogy();
  hRE->Draw();
  hRE->SetLineColor(kGreen);
  hLE->Draw("same");
  hLE->SetLineColor(kRed);

  c1->cd(2);
  gPad->SetGrid();
  hREminusLE->Draw();
  hREminusLE->SetLineColor(kBlue);
  TF1 *f1 = new TF1("f1", "gaus", -200, 200);
  hREminusLE->Fit("f1", "RWV");

  cout<<"Parameter 1: "<<f1->GetParameter(1)<<endl;

  c1->cd(3);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hLE_RE->Draw("colz");

 
  double result =  f1->GetParameter(1);

  delete hLE;
  delete hRE;
  delete hREminusLE;
  delete hLE_RE;
  delete f1;
  delete c1;
  delete t1;

  return result;
}



double DoMinimization(int MAXnum, int PaddleNumber)
{
  double minFac = 0.5;
  double maxFac = 1.5;
  const int MaxIter = 40;


  double tockeX[MaxIter];
  double tockeY[MaxIter];
  double tockeZ[MaxIter];
   
  double val = 1.0;
  int num = 0;
  double fac;
  
  while ( fabs(val)>0.01 && num < MaxIter )
  {
    
    fac = (minFac + maxFac)/2.0;
    val = CalculateFactor(fac, MAXnum, PaddleNumber);
    if (val < 0.0){
      minFac = fac;
    }
    else{ 
       maxFac = fac;
    }

    tockeX[num] = num;
    tockeY[num] = fac;
    tockeZ[num] = val+1.0;
    num++;

  }


  TCanvas* c2 = new TCanvas("c2","c2",1170,660);
  c2->Divide(2,1,0.001,0.001);
  c2->cd(1);
  gPad->SetGrid();
  TGraph *gr = new TGraph(num, tockeX, tockeY);
  gr->Draw("APL");
  gr->SetLineColor(kRed);

  TString figurename;
  figurename.Form("Mult. factor for E-paddle #%d",PaddleNumber);
  gr->SetTitle(figurename.Data());
  gr->GetXaxis()->SetTitle("Iteration [1]");
  gr->GetYaxis()->SetTitle("R-Mult. factor [1]");
 
  c2->cd(2);
  gPad->SetGrid();
  gPad->SetLogy();
  TGraph *gr2 = new TGraph(num, tockeX, tockeZ);
  gr2->Draw("APL");
  gr2->SetLineColor(kBlue);

  figurename.Form("Offset from center in E-paddle #%d",PaddleNumber);
  gr2->SetTitle(figurename.Data());
  gr2->GetXaxis()->SetTitle("Iteration [1]");
  gr2->GetYaxis()->SetTitle("1+Offset [ADC channel]");


  figurename.Form("figure_EPaddle_%d_OffsetMinimization.png",PaddleNumber);
  c2->Print(figurename.Data()); 


  cout<<"Final correction Factor: "<<fac<<endl;

  delete gr;
  delete gr2;
  delete c2;

  return fac;
  

}

double DoResults(double MulFactor, int MAXnum, int PaddleNumber)
{
  Double_t BB_tp_e_LE[24];
  Double_t BB_tp_e_RE[24];  
  Int_t Ndata_BB_tp_e_LE;
  Int_t Ndata_BB_tp_e_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];


  TChain *t1 =new TChain("tout");
  
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3488.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  assert(t1);  

  t1->SetBranchAddress( "BB.tp.e.LE", &BB_tp_e_LE);
  t1->SetBranchAddress( "BB.tp.e.RE", &BB_tp_e_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.e.LE", &Ndata_BB_tp_e_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.e.RE", &Ndata_BB_tp_e_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try); 


  const int NUM = 23;
  Double_t yCuts[NUM] = {-0.22, -0.2, -0.18, -0.16, -0.14, -0.12, -0.10, -0.08, -0.06, -0.04, -0.02, 
                          0.0, 0.02, 0.04, 0.06, 0.08,0.1,0.12, 0.14, 0.16, 0.18, 0.2, 0.22};

  TH1F *hLE[NUM];
  TH1F *hRE[NUM];
  TH1F *hREminusLE[NUM];
  TH2F *hLE_RE[NUM];
  TCanvas* cY[NUM];

  Double_t resY[NUM];
  Double_t resOffset[NUM];
  Double_t resSigmaY[NUM];
  Double_t resSigmaOffset[NUM];


  for (int M = 0; M<NUM; M++){

    int num = t1->GetEntries();
    cout<<"Number of entries : "<<num<<endl;
    double number = MAXnum;
    if (MAXnum<0) number = num;

    TString name;
    name.Form("hLE_YCut_%d",M);
    hLE[M] = new TH1F(name.Data(), name.Data(), 100,0,4000);
    
    name.Form("hRE_YCut_%d",M);
    hRE[M] = new TH1F(name.Data(), name.Data(), 100,0,4000);

    name.Form("hREminusLE_YCut_%d",M);
    hREminusLE[M] = new TH1F(name.Data(), name.Data(), 100,-700,700);

    name.Form("hLE_RE_YCut_%d",M);   
    hLE_RE[M] = new TH2F(name.Data(), name.Data(), 200,0,4000,200,0,4000);


    for (int i = 0; i<number; i++){
      t1->GetEntry(i);
      if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

      if (BB_tp_e_LE[PaddleNumber] < 500 || BB_tp_e_RE[PaddleNumber] < 500 ) continue;
      if ( fabs(BB_tp_try[0]+yCuts[M]) >0.01 ) continue;

      hLE[M]->Fill(BB_tp_e_LE[PaddleNumber]);
      hRE[M]->Fill(BB_tp_e_RE[PaddleNumber]);
      hREminusLE[M]->Fill(MulFactor*BB_tp_e_RE[PaddleNumber] - BB_tp_e_LE[PaddleNumber]);
      hLE_RE[M]->Fill(BB_tp_e_LE[PaddleNumber], BB_tp_e_RE[PaddleNumber]);
 
    }

    name.Form("CY_%d",M);
    cY[M] = new TCanvas(name.Data(),name.Data(),1170,660);
    cY[M]->Divide(3,1,0.001,0.001);
    cY[M]->cd(1);
    gPad->SetGrid();
    gPad->SetLogy();
    hRE[M]->Draw();
    hRE[M]->SetLineColor(kGreen);
    hLE[M]->Draw("same");
    hLE[M]->SetLineColor(kRed);

    cY[M]->cd(2);
    gPad->SetGrid();
    hREminusLE[M]->Draw();
    hREminusLE[M]->SetLineColor(kBlue);
    TF1 *f1 = new TF1("f1", "gaus", -700, 700);
    hREminusLE[M]->Fit("f1", "RWV");

    cout<<"Parameter <mean>: "<<f1->GetParameter(1)<<endl;
    cout<<"Parameter <sigma>: "<<f1->GetParameter(2)<<endl;

    resY[M] = yCuts[M];
    resOffset[M] = f1->GetParameter(1);
    resSigmaOffset[M] = f1->GetParameter(2);
    resSigmaY[M] = 0.01;

    cY[M]->cd(3);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hLE_RE[M]->Draw("colz");

    name.Form("figure_EPaddle_%d_YCut%d.png",PaddleNumber, M);
    cY[M]->Print(name.Data());

    delete hLE[M];
    delete hRE[M];
    delete hREminusLE[M];
    delete hLE_RE[M];
    delete f1;
    delete cY[M];
    
  }


  TCanvas* cRes = new TCanvas("cRes","cRes",1170,660);
  cRes->Divide(1,1,0.001,0.001);
  cRes->cd(1);
  gPad->SetGrid();
  TGraphErrors *gr = new TGraphErrors(NUM, resY, resOffset, resSigmaY, resSigmaOffset);
  gr->Draw("AP");
  gr->SetMarkerStyle(20);
  gr->SetMarkerColor(kRed);
 
  TString figurename;
  figurename.Form("Y Dependence for E-paddle #%d",PaddleNumber);
  gr->SetTitle(figurename.Data());
  gr->GetXaxis()->SetTitle("Y-offset [m]");
  gr->GetYaxis()->SetTitle("ADC-offset [channel]");
 
  figurename.Form("figure_EPaddle_%d_YDep.png",PaddleNumber);
  cRes->Print(figurename.Data());

/*
  delete gr;
  delete cRes;
  delete t1;
*/
}


void EplaneCalibration()
{

  //DoMinimization(-1, 4);
  DoResults(1.00142, -1, 4);

/*
  ofstream outFile;
  outFile.open("EplaneCorrectionFactors.dat");

  for (int i = 0; i<24; i++){
    double fac = DoMinimization(-1, i);
    outFile<<i<<"  "<<fac<<endl;
    DoResults(fac, -1, i);
  }

  outFile.close();

*/

}

