#include "Riostream.h"
#include "TDatime.h"
#include "TPolyMarker.h"
#include "TPolyLine.h"
#include "TLine.h"
#include "TCanvas.h"
#include "TGraph.h"
#include "TGraphErrors.h"
#include "TAxis.h"
#include "TH1.h"
#include "TH2.h"
#include "TLegend.h"  
#include "TFile.h"
#include "TTree.h"
#include "TStyle.h"
#include "TChain.h"
#include "TMarker.h"
#include "TROOT.h"
#include "TF1.h"
#include "TProfile.h"
#include "TStyle.h"

void HallA_style() {
  gROOT->SetStyle("Plain");
  gStyle->SetPaperSize(TStyle::kUSLetter);
  gStyle->SetPaperSize(18,22);
  gStyle->SetOptFit(1111);
  gStyle->SetPalette(1);
  gStyle->SetNdivisions(505);
  
  gStyle->SetCanvasColor(10);
  gStyle->SetPadTopMargin(.05);
  gStyle->SetPadLeftMargin(.1);
  gStyle->SetPadRightMargin(.1);
  gStyle->SetPadBottomMargin(.10);
  gStyle->SetTitleYOffset(1.3);
  gStyle->SetLabelFont(42,"X");
  gStyle->SetLabelFont(42,"Y");

  // prepare gStyle to be useful
  //   1 = solid
  //   2 = long dash (30 10)
  //   3 = dotted (4 8)
  //   4 = dash-dot (15 12 4 12)
  //   5 = short dash ( 15 15 )
  //   6 = dash-dot-dot 
  gStyle->SetLineStyleString(1,"[]");
  gStyle->SetLineStyleString(2,"[30 10]");
  gStyle->SetLineStyleString(3,"[4 8]");
  gStyle->SetLineStyleString(4,"[15 12 4 12]");
  gStyle->SetLineStyleString(5,"[15 15]");
  gStyle->SetLineStyleString(6,"[15 12 4 12 4 12]");
  gStyle->SetLabelSize(0.045,"X");
  gStyle->SetTitleOffset(1.3,"X");
  gStyle->SetLabelSize(0.045,"Y");
  gStyle->SetTitleOffset(1.3,"Y");
  gStyle->SetNdivisions(505,"Y");
  gStyle->SetOptDate(0);
  gStyle->SetDateY(.98);
  gStyle->SetStripDecimals(kFALSE);
  gStyle->SetEndErrorSize(7);
}



double CalculateFactor(double MulFactor, int MAXnum, int PaddleNumber )
{
  Double_t BB_tp_de_LE[24];
  Double_t BB_tp_de_RE[24];  
  Int_t Ndata_BB_tp_de_LE;
  Int_t Ndata_BB_tp_de_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];
  Double_t BB_tp_de_hit_bar[100];
  Double_t BB_tp_trHitIndex[100];


  TChain *t1 =new TChain("tout");
  
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3488.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3265.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3608.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3270.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3271.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3272.root");
  assert(t1);  

  t1->SetBranchAddress( "BB.tp.de.LE", &BB_tp_de_LE);
  t1->SetBranchAddress( "BB.tp.de.RE", &BB_tp_de_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.LE", &Ndata_BB_tp_de_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.RE", &Ndata_BB_tp_de_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try); 
  t1->SetBranchAddress( "BB.tp.de.hit_bar", &BB_tp_de_hit_bar);  
  t1->SetBranchAddress( "BB.tp.trHitIndex", &BB_tp_trHitIndex);




  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;
  double number = MAXnum;
  if (MAXnum<0) number = num;

  TH1F *hLE = new TH1F("hLE", "hLE", 100,0,4000);
  TH1F *hRE = new TH1F("hRE", "hRE", 100,0,4000);
  TH1F *hREminusLE = new TH1F("hREminusLE", "hREminusLE", 100,-300,300);

  TH2F *hLE_RE = new TH2F("hLE_RE", "hLE_RE", 200,0,4000,200,0,4000);


  for (int i = 0; i<number; i++)
  {
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int dEplaneHit = (int)BB_tp_de_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];
   
    if (dEplaneHit != PaddleNumber) continue;

    if (BB_tp_de_LE[PaddleNumber] < 150 || BB_tp_de_RE[PaddleNumber] < 150 ) continue;
    if ( fabs(FpY) >0.01 ) continue;

    hLE->Fill(BB_tp_de_LE[PaddleNumber]);
    hRE->Fill(BB_tp_de_RE[PaddleNumber]);
    hREminusLE->Fill(MulFactor*BB_tp_de_RE[PaddleNumber] - BB_tp_de_LE[PaddleNumber]);
    hLE_RE->Fill(BB_tp_de_LE[PaddleNumber], BB_tp_de_RE[PaddleNumber]);
 
  }

  TCanvas* c1 = new TCanvas("c1","c1",1170,660);
  c1->Divide(3,1,0.001,0.001);
  c1->cd(1);
  gPad->SetGrid();
  gPad->SetLogy();
  hRE->Draw();
  hRE->SetLineColor(kGreen);
  hLE->Draw("same");
  hLE->SetLineColor(kRed);

  c1->cd(2);
  gPad->SetGrid();
  hREminusLE->Draw();
  hREminusLE->SetLineColor(kBlue);
  TF1 *f1 = new TF1("f1", "gaus", -200, 200);
  hREminusLE->Fit("f1", "RWV");

  cout<<"Parameter 1: "<<f1->GetParameter(1)<<endl;

  c1->cd(3);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hLE_RE->Draw("colz");

 
  double result =  f1->GetParameter(1);


  delete hLE;
  delete hRE;
  delete hREminusLE;
  delete hLE_RE;
  delete f1;
  delete c1;
  delete t1;
  
  

  return result;
}



double DoMinimization(int MAXnum, int PaddleNumber)
{
  double minFac = 0.5;
  double maxFac = 1.6;
  const int MaxIter = 40;


  double tockeX[MaxIter];
  double tockeY[MaxIter];
  double tockeZ[MaxIter];
   
  double val = 1.0;
  int num = 0;
  double fac;
  
  while ( fabs(val)>0.01 && num < MaxIter )
  {
    
    fac = (minFac + maxFac)/2.0;
    val = CalculateFactor(fac, MAXnum, PaddleNumber);
    if (val < 0.0){
      minFac = fac;
    }
    else{ 
       maxFac = fac;
    }

    tockeX[num] = num;
    tockeY[num] = fac;
    tockeZ[num] = val+1.0;
    num++;

  }


  TCanvas* c2 = new TCanvas("c2","c2",1170,660);
  c2->Divide(2,1,0.001,0.001);
  c2->cd(1);
  gPad->SetGrid();
  TGraph *gr = new TGraph(num, tockeX, tockeY);
  gr->Draw("APL");
  gr->SetLineColor(kRed);

  TString figurename;
  figurename.Form("Mult. factor for dE-paddle #%d",PaddleNumber);
  gr->SetTitle(figurename.Data());
  gr->GetXaxis()->SetTitle("Iteration [1]");
  gr->GetYaxis()->SetTitle("R-Mult. factor [1]");
 
  c2->cd(2);
  gPad->SetGrid();
  gPad->SetLogy();
  TGraph *gr2 = new TGraph(num, tockeX, tockeZ);
  gr2->Draw("APL");
  gr2->SetLineColor(kBlue);

  figurename.Form("Offset from center in dE-paddle #%d",PaddleNumber);
  gr2->SetTitle(figurename.Data());
  gr2->GetXaxis()->SetTitle("Iteration [1]");
  gr2->GetYaxis()->SetTitle("1+Offset [ADC channel]");


  figurename.Form("figure_dEPaddle_%d_OffsetMinimization.png",PaddleNumber);
  c2->Print(figurename.Data()); 


  cout<<"Final correction Factor: "<<fac<<endl;
/*
  delete gr;
  delete gr2;
  delete c2;
*/
  return fac;
  

}

double DoResults(double MulFactor, int MAXnum, int PaddleNumber, double *FitParameters)
{
  Double_t BB_tp_de_LE[24];
  Double_t BB_tp_de_RE[24];  
  Int_t Ndata_BB_tp_de_LE;
  Int_t Ndata_BB_tp_de_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];
  Double_t BB_tp_de_hit_bar[100];
  Double_t BB_tp_trHitIndex[100];



  TChain *t1 =new TChain("tout");
  
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3270.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3271.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3272.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3273.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3274.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3275.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3276.root");

  assert(t1);  

  t1->SetBranchAddress( "BB.tp.de.LE", &BB_tp_de_LE);
  t1->SetBranchAddress( "BB.tp.de.RE", &BB_tp_de_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.LE", &Ndata_BB_tp_de_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.RE", &Ndata_BB_tp_de_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try); 
  t1->SetBranchAddress( "BB.tp.de.hit_bar", &BB_tp_de_hit_bar);  
  t1->SetBranchAddress( "BB.tp.trHitIndex", &BB_tp_trHitIndex);


  const int NUM = 23;
  Double_t yCuts[NUM] = {-0.22, -0.2, -0.18, -0.16, -0.14, -0.12, -0.10, -0.08, -0.06, -0.04, -0.02, 
                          0.0, 0.02, 0.04, 0.06, 0.08,0.1,0.12, 0.14, 0.16, 0.18, 0.2, 0.22};

  TH1F *hLE[NUM];
  TH1F *hRE[NUM];
  TH1F *hREminusLE[NUM];
  TH2F *hLE_RE[NUM];
  TCanvas* cY[NUM];
  TF1 *f1Y[NUM] ;

  Double_t resY[NUM];
  Double_t resOffset[NUM];
  Double_t resSigmaY[NUM];
  Double_t resSigmaOffset[NUM];


  for (int M = 0; M<NUM; M++){

    TString name;
    name.Form("hLE_YCut_%d",M);
    hLE[M] = new TH1F(name.Data(), name.Data(), 100,0,4000);
    
    name.Form("hRE_YCut_%d",M);
    hRE[M] = new TH1F(name.Data(), name.Data(), 100,0,4000);

    name.Form("hREminusLE_YCut_%d",M);
    hREminusLE[M] = new TH1F(name.Data(), name.Data(), 100,-900,900);

    name.Form("hLE_RE_YCut_%d",M);   
    hLE_RE[M] = new TH2F(name.Data(), name.Data(), 200,0,4000,200,0,4000);
 

  }


 
  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;
  double number = MAXnum;
  if (MAXnum<0) number = num;

  for (int i = 0; i<number; i++){
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int dEplaneHit = (int)BB_tp_de_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];

    if (dEplaneHit!=PaddleNumber) continue;

    for (int M = 0; M<NUM; M++){
      if (BB_tp_de_LE[PaddleNumber] < 150 || BB_tp_de_RE[PaddleNumber] < 150 ) continue;
      if ( fabs(FpY - yCuts[M]) >0.01 ) continue;

      hLE[M]->Fill(BB_tp_de_LE[PaddleNumber]);
      hRE[M]->Fill(BB_tp_de_RE[PaddleNumber]);
      hREminusLE[M]->Fill(MulFactor*BB_tp_de_RE[PaddleNumber] - BB_tp_de_LE[PaddleNumber]);
      hLE_RE[M]->Fill(BB_tp_de_LE[PaddleNumber], BB_tp_de_RE[PaddleNumber]);
    }
 
  }

  for (int M = 0; M<NUM; M++){
    TString name;
    name.Form("CY_%d",M);
    cY[M] = new TCanvas(name.Data(),name.Data(),1170,660);
    cY[M]->Divide(3,1,0.001,0.001);
    cY[M]->cd(1);
    gPad->SetGrid();
    gPad->SetLogy();
    hRE[M]->Draw();
    hRE[M]->SetLineColor(kGreen);
    hLE[M]->Draw("same");
    hLE[M]->SetLineColor(kRed);

    cY[M]->cd(2);
    gPad->SetGrid();
    hREminusLE[M]->Draw();
    hREminusLE[M]->SetLineColor(kBlue);
    name.Form("f1Y_%d",M);
    f1Y[M] = new TF1(name.Data(), "gaus", -900, 900);
    hREminusLE[M]->Fit(name.Data(), "RWV");

    cout<<"Parameter <mean>: "<<f1Y[M]->GetParameter(1)<<endl;
    cout<<"Parameter <sigma>: "<<f1Y[M]->GetParameter(2)<<endl;

    resY[M] = yCuts[M];
    resOffset[M] = f1Y[M]->GetParameter(1);
    resSigmaOffset[M] = f1Y[M]->GetParameter(2);
    resSigmaY[M] = 0.01;

    cY[M]->cd(3);
    gPad->SetGrid();
    gStyle->SetPalette(1);
    hLE_RE[M]->Draw("colz");

    name.Form("figure_dEPaddle_%d_YCut%d.png",PaddleNumber, M);
    cY[M]->Print(name.Data());

  }
  

  TCanvas* cRes = new TCanvas("cRes","cRes",1170,660);
  cRes->Divide(1,1,0.001,0.001);
  cRes->cd(1);
  gPad->SetGrid();
  TGraphErrors *gr = new TGraphErrors(NUM, resY, resOffset, resSigmaY, resSigmaOffset);
  gr->Draw("AP");
  gr->SetMarkerStyle(20);
  gr->SetMarkerColor(kRed);
 
  TString figurename;
  figurename.Form("Y Dependence for dE-paddle #%d",PaddleNumber);
  gr->SetTitle(figurename.Data());
  gr->GetXaxis()->SetTitle("Y-offset [m]");
  gr->GetYaxis()->SetTitle("ADC-offset [channel]");

  TF1 *f1 = new TF1("f1", "[0] + [1]*x + [2]*x^3", -0.25, 0.25);
  gr->Fit("f1","RV");

  FitParameters[0] = f1->GetParameter(0);
  FitParameters[1] = f1->GetParameter(1);
  FitParameters[2] = f1->GetParameter(2);

  figurename.Form("figure_dEPaddle_%d_YDep.png",PaddleNumber);
  cRes->Print(figurename.Data());


  for (int M = 0; M<NUM; M++){
    delete hLE[M];
    delete hRE[M];
    delete hREminusLE[M];
    delete hLE_RE[M];
    delete f1Y[M];
    delete cY[M];
  }
/*
  delete f1;
  delete gr;
  delete cRes;
  delete t1;
*/

}


double MeanYDependence(double MulFactor, int MAXnum, int PaddleNumber, double *ResParams)
{
  Double_t BB_tp_de_LE[24];
  Double_t BB_tp_de_RE[24];  
  Int_t Ndata_BB_tp_de_LE;
  Int_t Ndata_BB_tp_de_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];
  Double_t BB_tp_de_hit_bar[100];
  Double_t BB_tp_trHitIndex[100];


  TChain *t1 =new TChain("tout");

  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3270.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3271.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3272.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3273.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3274.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3275.root");
  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3276.root");
  assert(t1);  

  t1->SetBranchAddress( "BB.tp.de.LE", &BB_tp_de_LE);
  t1->SetBranchAddress( "BB.tp.de.RE", &BB_tp_de_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.LE", &Ndata_BB_tp_de_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.RE", &Ndata_BB_tp_de_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try);
  t1->SetBranchAddress( "BB.tp.de.hit_bar", &BB_tp_de_hit_bar);  
  t1->SetBranchAddress( "BB.tp.trHitIndex", &BB_tp_trHitIndex);


  const int NUM = 23;
  Double_t yCuts[NUM] = {-0.22, -0.2, -0.18, -0.16, -0.14, -0.12, -0.10, -0.08, -0.06, -0.04, -0.02, 
                          0.0, 0.02, 0.04, 0.06, 0.08,0.1,0.12, 0.14, 0.16, 0.18, 0.2, 0.22};

  Double_t CutSize = 0.01;

  Double_t resY[NUM];
  Double_t resADCMean[NUM];
  Double_t resSigmaY[NUM];
  Double_t resSigmaADCMean[NUM];
  TH1F *hADCdE_FpYDep[NUM];
  TCanvas *cY[NUM] ;
  TF1 *fitfun[NUM]; 


  for(int i = 0; i<NUM; i++){

    TString name;
    name.Form("hADCdE_FpYDep_%d",i);
    hADCdE_FpYDep[i] = new TH1F(name.Data(), name.Data(), 100,0,2500);
    
  }


  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;
  double number = MAXnum;
  if (MAXnum<0) number = num;

  for (int i = 0; i<number; i++){
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int dEplaneHit = (int)BB_tp_de_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];

    if (dEplaneHit!=PaddleNumber) continue;
    
    for(int j = 0; j<NUM; j++){ 
      if ( fabs(FpY-yCuts[j]) > CutSize ) continue;
      hADCdE_FpYDep[j]->Fill(sqrt(MulFactor*BB_tp_de_RE[PaddleNumber] * BB_tp_de_LE[PaddleNumber]));

    }


  }

  for (int i = 0; i<NUM; i++){
    TString name;
    name.Form("CY_%d",i);  
    cY[i] = new TCanvas(name.Data(),name.Data(),1170,660);
    cY[i]->Divide(1,1,0.001,0.001);
    cY[i]->cd(1);
    gPad->SetGrid(); 
    hADCdE_FpYDep[i]->Draw();
    name.Form("fitfun_%d",i);  
    //fitfun[i] = new TF1(name.Data(), "gaus", 0, 2500);
    fitfun[i] = new TF1(name.Data(), "landau", 0, 2500);
    hADCdE_FpYDep[i]->Fit(name.Data(), "RWV");

    resY[i] = yCuts[i];
    resADCMean[i] = fitfun[i]->GetParameter(1);
    resSigmaY[i] = CutSize;
    resSigmaADCMean[i]= fitfun[i]->GetParError(1);
    //resSigmaADCMean[i]= fitfun[i]->GetParError(1);

    name.Form("figure_dEPaddle_%d_YCorrectionFactor_YCut_%d.png",PaddleNumber, i);
    cY[i]->Print(name.Data());
    
    delete fitfun[i];
    delete hADCdE_FpYDep[i];
    delete cY[i];
  }
  

  TCanvas* cRes = new TCanvas("cRes","cRes",1170,660);
  cRes->Divide(1,1,0.001,0.001);
  cRes->cd(1);
  gPad->SetGrid();
  TGraphErrors *gr = new TGraphErrors(NUM, resY, resADCMean, resSigmaY, resSigmaADCMean);
  gr->Draw("AP");
  gr->SetMarkerStyle(20);
  gr->SetMarkerColor(kRed);
  TF1 *f1 = new TF1("f1", "[0] + [1]*x + [2]*x^2 + [3]*x^3 + [4]*x^4 ", -0.25, 0.25);
  gr->Fit("f1","RWV");

  ResParams[0] = f1->GetParameter(0);
  ResParams[1] = f1->GetParameter(1);
  ResParams[2] = f1->GetParameter(2);
  ResParams[3] = f1->GetParameter(3);
  ResParams[4] = f1->GetParameter(4);

  TString figurename;
  figurename.Form("Y Dependence of <ADC> for dE-paddle #%d",PaddleNumber);
  gr->SetTitle(figurename.Data());
  gr->GetXaxis()->SetTitle("Y-offset [m]");
  gr->GetYaxis()->SetTitle("<ADC> [channel]");
 
  figurename.Form("figure_dEPaddle_%d_YCorrectionFactorsFit.png",PaddleNumber);
  cRes->Print(figurename.Data());
/*
  delete gr;
  delete f1;
  delete cRes;
  delete t1;

*/
    
  TH2F *hADCdEvsFpYPaddle = new TH2F("hADCdEvsFpYPaddle", "hADCdEvsFpYPaddle", 500,-0.3,0.3,500,0,1500);
  TH2F *hADCdEvsFpYPaddleCorrected = new TH2F("hADCdEvsFpYPaddleCorrected", "hADCdEvsFpYPaddleCorrected", 500,-0.3,0.3,500,0,1500);

  for (int i = 0; i<number; i++){
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  Equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int dEplaneHit = (int)BB_tp_de_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];

    if (dEplaneHit!=PaddleNumber) continue;
    
    double dEmeanADCValue = sqrt(MulFactor*BB_tp_de_RE[dEplaneHit]*BB_tp_de_LE[dEplaneHit]);
    double dEmeanADCValueCorrected = sqrt(MulFactor*BB_tp_de_RE[dEplaneHit]*BB_tp_de_LE[dEplaneHit]) - 
    ResParams[1]*pow(FpY,1) - ResParams[2]*pow(FpY,2) - ResParams[3]*pow(FpY,3) - ResParams[4]*pow(FpY,4);

    hADCdEvsFpYPaddle->Fill(FpY,dEmeanADCValue);
    hADCdEvsFpYPaddleCorrected->Fill(FpY,dEmeanADCValueCorrected);
   
  }

  TLine *LimitLine = new TLine(-0.3,  ResParams[0], 0.3,  ResParams[0]);
  LimitLine->SetLineColor(kRed);
  LimitLine->SetLineWidth(1.5);
  

  TCanvas *cCorr = new TCanvas("cCorr","cCorr",660,660);
  cCorr->Divide(1,2,0.001,0.001);
  cCorr->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hADCdEvsFpYPaddle->Draw("colz");
  LimitLine->Draw("same");
  cCorr->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hADCdEvsFpYPaddleCorrected->Draw("colz");
  LimitLine->Draw("same");

  figurename.Form("figure_dEPaddle_%d_YCorrected.png",PaddleNumber);
  cCorr->Print(figurename.Data());




    
}


double MeanYDependenceLowLimit(double MulFactor, int MAXnum, int PaddleNumber, double *ResParams)
{
  Double_t BB_tp_de_LE[24];
  Double_t BB_tp_de_RE[24];  

  Double_t BB_tp_e_LE[24];
  Double_t BB_tp_e_RE[24];  
  Double_t BB_tp_e_hit_bar[100];

  Int_t Ndata_BB_tp_de_LE;
  Int_t Ndata_BB_tp_de_RE;  
  Double_t BB_tp_trx[100];
  Double_t BB_tp_try[100];
  Double_t BB_tp_de_hit_bar[100];
  Double_t BB_tp_trHitIndex[100];


  TChain *t1 =new TChain("tout");

  //t1->Add("./ROOTResults/eed_Test_GoodEvents_3151.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3270.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3271.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3272.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3273.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3274.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3275.root");
  t1->Add("./ROOTResults/eed_Test_GoodEvents_3276.root");

  assert(t1);  

  t1->SetBranchAddress( "BB.tp.de.LE", &BB_tp_de_LE);
  t1->SetBranchAddress( "BB.tp.de.RE", &BB_tp_de_RE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.LE", &Ndata_BB_tp_de_LE);
  t1->SetBranchAddress( "Ndata.BB.tp.de.RE", &Ndata_BB_tp_de_RE);
  t1->SetBranchAddress( "BB.tp.trx", &BB_tp_trx);
  t1->SetBranchAddress( "BB.tp.try", &BB_tp_try);
  t1->SetBranchAddress( "BB.tp.de.hit_bar", &BB_tp_de_hit_bar);  
  t1->SetBranchAddress( "BB.tp.trHitIndex", &BB_tp_trHitIndex);
  t1->SetBranchAddress( "BB.tp.e.LE", &BB_tp_e_LE);
  t1->SetBranchAddress( "BB.tp.e.RE", &BB_tp_e_RE);
  t1->SetBranchAddress( "BB.tp.e.hit_bar", &BB_tp_e_hit_bar);  

  const int NUM = 23;
  Double_t yCuts[NUM] = {-0.22, -0.2, -0.18, -0.16, -0.14, -0.12, -0.10, -0.08, -0.06, -0.04, -0.02, 
                          0.0, 0.02, 0.04, 0.06, 0.08,0.1,0.12, 0.14, 0.16, 0.18, 0.2, 0.22};

  Double_t CutSize = 0.01;

  Double_t resY[NUM];
  Double_t resADCMean[NUM];
  Double_t resSigmaY[NUM];
  Double_t resSigmaADCMean[NUM];
  TH1F *hADCdE_FpYDep[NUM];
  TCanvas *cY[NUM] ;
  TF1 *fitfun[NUM]; 


  for(int i = 0; i<NUM; i++){

    TString name;
    name.Form("hADCdE_FpYDep_%d",i);
    hADCdE_FpYDep[i] = new TH1F(name.Data(), name.Data(), 100,0,2500);
    
  }


  int num = t1->GetEntries();
  cout<<"Number of entries : "<<num<<endl;
  double number = MAXnum;
  if (MAXnum<0) number = num;

  for (int i = 0; i<number; i++){
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int dEplaneHit = (int)BB_tp_de_hit_bar[trHitIndex];
    int EplaneHit = (int)BB_tp_e_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];

    if (dEplaneHit!=PaddleNumber || EplaneHit!=PaddleNumber) continue;
    
    for(int j = 0; j<NUM; j++){ 
      double rawEADC = sqrt(BB_tp_e_RE[PaddleNumber] * BB_tp_e_LE[PaddleNumber]);
      if (fabs(rawEADC-1000)>100) continue;
      if ( fabs(FpY-yCuts[j]) > CutSize ) continue;
      hADCdE_FpYDep[j]->Fill(sqrt(MulFactor*BB_tp_de_RE[PaddleNumber] * BB_tp_de_LE[PaddleNumber]));

    }


  }

  for (int i = 0; i<NUM; i++){
    TString name;
    name.Form("CY_%d",i);  
    cY[i] = new TCanvas(name.Data(),name.Data(),1170,660);
    cY[i]->Divide(1,1,0.001,0.001);
    cY[i]->cd(1);
    gPad->SetGrid(); 
    hADCdE_FpYDep[i]->Draw();
    name.Form("fitfun_%d",i);  
    fitfun[i] = new TF1(name.Data(), "gaus", 0, 2500);
    hADCdE_FpYDep[i]->Fit(name.Data(), "RWV");

    resY[i] = yCuts[i];
    resADCMean[i] = fitfun[i]->GetParameter(1);
    resSigmaY[i] = CutSize;
    resSigmaADCMean[i]= fitfun[i]->GetParError(1);
    //resSigmaADCMean[i]= fitfun[i]->GetParError(1);

    name.Form("figure_dEPaddle_%d_YCorrectionFactor_YCut_%d.png",PaddleNumber, i);
    cY[i]->Print(name.Data());
    
    delete fitfun[i];
    delete hADCdE_FpYDep[i];
    delete cY[i];
  }
  

  TCanvas* cRes = new TCanvas("cRes","cRes",1170,660);
  cRes->Divide(1,1,0.001,0.001);
  cRes->cd(1);
  gPad->SetGrid();
  TGraphErrors *gr = new TGraphErrors(NUM, resY, resADCMean, resSigmaY, resSigmaADCMean);
  gr->Draw("AP");
  gr->SetMarkerStyle(20);
  gr->SetMarkerColor(kRed);
  TF1 *f1 = new TF1("f1", "[0] + [1]*x^2 + [2]*x^4 ", -0.25, 0.25);
  gr->Fit("f1","RWV");

  ResParams[0] = f1->GetParameter(0);
  ResParams[1] = f1->GetParameter(1);
  ResParams[2] = f1->GetParameter(2);

  TString figurename;
  figurename.Form("Y Dependence of <ADC> for dE-paddle #%d",PaddleNumber);
  gr->SetTitle(figurename.Data());
  gr->GetXaxis()->SetTitle("Y-offset [m]");
  gr->GetYaxis()->SetTitle("<ADC> [channel]");
 
  figurename.Form("figure_dEPaddle_%d_YCorrectionFactorsFit.png",PaddleNumber);
  cRes->Print(figurename.Data());
/*
  delete gr;
  delete f1;
  delete cRes;
  delete t1;

*/
    
  TH2F *hADCdEvsFpYPaddle = new TH2F("hADCdEvsFpYPaddle", "hADCdEvsFpYPaddle", 500,-0.3,0.3,500,0,1500);
  TH2F *hADCdEvsFpYPaddleCorrected = new TH2F("hADCdEvsFpYPaddleCorrected", "hADCdEvsFpYPaddleCorrected", 500,-0.3,0.3,500,0,1500);

  for (int i = 0; i<number; i++){
    t1->GetEntry(i);
    if ((i%1000==0) && i>0) cout<<i<<"  Equals to: "<<i/number*100.0<<"%"<<endl; 

    int trHitIndex = (int) BB_tp_trHitIndex[0];
    int dEplaneHit = (int)BB_tp_de_hit_bar[trHitIndex];
    double FpY = BB_tp_try[0];

    if (dEplaneHit!=PaddleNumber) continue;
    
    double dEmeanADCValue = sqrt(MulFactor*BB_tp_de_RE[dEplaneHit]*BB_tp_de_LE[dEplaneHit]);
    double dEmeanADCValueCorrected = sqrt(MulFactor*BB_tp_de_RE[dEplaneHit]*BB_tp_de_LE[dEplaneHit]) - 
    ResParams[1]*pow(FpY,2) - ResParams[2]*pow(FpY,4);

    hADCdEvsFpYPaddle->Fill(FpY,dEmeanADCValue);
    hADCdEvsFpYPaddleCorrected->Fill(FpY,dEmeanADCValueCorrected);
   
  }

  TLine *LimitLine = new TLine(-0.3,  ResParams[0], 0.3,  ResParams[0]);
  LimitLine->SetLineColor(kRed);
  LimitLine->SetLineWidth(1.5);
  

  TCanvas *cCorr = new TCanvas("cCorr","cCorr",660,660);
  cCorr->Divide(1,2,0.001,0.001);
  cCorr->cd(1);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hADCdEvsFpYPaddle->Draw("colz");
  LimitLine->Draw("same");
  cCorr->cd(2);
  gPad->SetGrid();
  gStyle->SetPalette(1);
  hADCdEvsFpYPaddleCorrected->Draw("colz");
  LimitLine->Draw("same");

  figurename.Form("figure_dEPaddle_%d_YCorrected.png",PaddleNumber);
  cCorr->Print(figurename.Data());




    
}




void dEplaneCalibration()
{
  HallA_style();


  
  //DoMinimization(-1, 7);


/*
  double FitParameters[3];
  DoResults(0.98266, -1, 7, FitParameters);
  cout<<"Fit parameters: 1: "<<FitParameters[0]<<", y: "<<FitParameters[1]<<", y^3: "<<FitParameters[2]<<endl;
*/

 double CorrectionParameters[5];
 MeanYDependence(1.0, -1, 7, CorrectionParameters);
 //MeanYDependenceLowLimit(1.01529, -1, 1, CorrectionParameters);
 cout<<"Correction parameters: [1]: "<<CorrectionParameters[0]<<", [y^1]: "<<CorrectionParameters[1]<<", [y^2]: "<<CorrectionParameters[2]<<", [y^3]: "<<CorrectionParameters[3]<<", [y^4]: "<<CorrectionParameters[4]<<endl;


/*
  ofstream outFile;
  outFile.open("dEplaneCorrectionFactors.dat");

 

  for (int i = 4; i<17; i++){
    double fac = DoMinimization(-1, i);
    cout<<"Factor for paddle: "<<i<<" is: "<<fac<<endl;
    double FitParameters[3] = {-1,-1,-1};
    DoResults(fac, -1, i, FitParameters);
    cout<<"Fit parameters: 1: "<<FitParameters[0]<<", y: "<<FitParameters[1]<<", y^3: "<<FitParameters[2]<<endl;
    double CorrectionParameters[2]={-1,-1};
    MeanYDependence(fac, -1, i, CorrectionParameters);
    cout<<"Correction parameters: y: "<<CorrectionParameters[0]<<", y^2: "<<CorrectionParameters[1]<<endl;

    outFile<<i<<"  "<<fac<<"  "<<FitParameters[0]<<"  "<<FitParameters[1]<<"  "<<FitParameters[2]<<"  "<<CorrectionParameters[0]<<"  "<<CorrectionParameters[1]<<endl;

  }


  outFile.close();
*/

}

